@@ -36,6 +36,14 @@ sub to_gray_rgb    { return from_rgb( (to_gray_byte($_[0])) x 3 ) }
 sub from_gray_byte { return 0x10101 * ($_[0] & 0xff) }
 sub premultiply { from_rgb( map { int($_ * $_[1] / 255) } to_rgb($_[0]) ) }
 
+sub distance
+{
+	my @x = to_rgb($_[0]);
+	my @y = to_rgb($_[1]);
+	$x[$_] -= $y[$_] for 0..2;
+	return sqrt( $x[0]*$x[0] + $x[1]*$x[1] + $x[2]*$x[2]);
+}
+
 package
     ci; *AUTOLOAD =  \&Prima::Const::AUTOLOAD;	# color indices
 package
@@ -75,15 +83,63 @@ package
 package
     le; *AUTOLOAD =  \&Prima::Const::AUTOLOAD;	# line ends
 
-sub Arrow { [
-	conic => [1,0,2.5,0,2.5,-0.5],
+use constant Arrow => [
+	conic => [1,0,1.5,0,2.5,-0.5],
 	line  => [0,2.5],
-	conic => [-2.5,-0.5,-2.5,0,-1,0]
-] }
+	conic => [-2.5,-0.5,-1.5,0,-1,0]
+];
 
-sub Rhombus { [
-	line  => [1,0,1.5,1,0,4,-1.5,1,-1,0],
-] }
+use constant Cusp => [
+	line  => [0,2],
+];
+
+use constant InvCusp => [
+	line  => [1,2,0,0,-1,2],
+];
+
+use constant Spearhead => [
+	line  => [1.5,1,0,4,-1.5,1],
+];
+
+use constant RoundRect => [
+	conic => [0,-1.5,1.5,-1.5,1.5,0,1.5,1.5,0,1.5,-1.5,1.5,-1.5,0,-1.5,-1.5,0,-1.5],
+];
+
+use constant Rect => [
+	line  => [0,-1.5,1.5,-1.5,1.5,1.5,-1.5,1.5,-1.5,-1.5,0,-1.5],
+];
+
+use constant Knob => [
+	conic => [0,-1.5,1.5,-1.5,1.5,0],
+	conic => [1.5,0,1.5,1.5,0,1.5],
+	conic => [0,1.5,-1.5,1.5,-1.5,0],
+	conic => [-1.5,0,-1.5,-1.5,0,-1.5],
+];
+
+use constant Tail => [
+	line  => [ 2,1,2,2.25,0,1.75,-2,2.25,-2,1 ]
+];
+
+sub transform
+{
+	my ( $le, $matrix ) = @_;
+	unless ( ref($le) ) {
+		my $can = __PACKAGE__->can($le);
+		return $le unless $can;
+		$le = $can->();
+	}
+
+	my @new;
+	for ( my $i = 0; $i < @$le; $i += 2 ) {
+		push @new, $$le[$i], Prima::matrix::transform($matrix, $$le[$i+1]);
+	}
+	return \@new;
+}
+
+sub scale { transform( $_[0], [ $_[1],0,0,$_[2] // $_[1],0,0 ] ) }
+
+package
+    lei; *AUTOLOAD =  \&Prima::Const::AUTOLOAD;	# line end indexes
 
 package
     lj; *AUTOLOAD =  \&Prima::Const::AUTOLOAD;	# line joins
@@ -317,6 +373,30 @@ See L<Prima::gp_problems/Colors>
 	cl::SysFlag  - indirect color constant bit set
 	cl::SysMask  - indirect color constant bit clear mask
 
+=item Color functions
+
+=over
+
+=item from_rgb R8,G8,B8 -> RGB24
+
+=item to_rgb   RGB24 -> R8,G8,B8
+
+=item from_bgr B8,G8,R8 -> RGB24
+
+=item to_bgr   RGB24 -> B8,G8,R8
+
+=item to_gray_byte RGB24 -> GRAY8
+
+=item to_gray_rgb  RGB24 -> GRAY24
+
+=item from_gray_byte GRAY8 -> GRAY24
+
+=item premultiply RGB24,A8 -> RGB24
+
+=item distance RGB24,RGB24 -> distance between colors
+
+=back
+
 =back
 
 =head2 cm::  - commands
@@ -430,13 +510,13 @@ Returns a C<cr::> constant corresponding to the C<ACTION>
 
 =item to_one_action ACTIONS
 
-Selects a best single action from combination of allowes C<ACTIONS>
+Selects a best single action from combination of allowed C<ACTIONS>
 
 =item keymod ACTION
 
-Returns a C<km::> keyboard modifier constant the C<ACTION> will be selected,
-when possible, if the user presses that modifier. Return 0 for C<dnd::Copy>
-that is a standard action to be performed without any modifiers.
+Returns a C<km::> keyboard modifier constant the C<ACTION> will be expected to
+start on if the user presses that modifier during a DND session. Returns 0 for
+C<dnd::Copy> that is the standard action to be performed without any modifiers.
 
 =back
 
@@ -640,6 +720,31 @@ See L<Prima::Drawable/lineEnd>
 	le::Square
 	le::Round
 
+	le::Arrow
+	le::Cusp
+	le::InvCusp
+	le::Knob
+	le::Rect
+	le::RoundRect
+	le::Spearhead
+	le::Tail
+
+Functions:
+
+	le::transform($matrix)
+	le::scale($scalex, [$scaley = $scalex])
+
+=head2 lei::  - line end indexes
+
+	lei::LineTail
+	lei::LineHead
+	lei::ArrowTail
+	lei::ArrowHead
+	lei::Max
+	lei::Only
+
+See L<Prima::Drawable/lineEndIndex>
+
 =head2 lj::  - line join styles
 
 See L<Prima::Drawable/lineJoin>
@@ -1027,6 +1132,25 @@ Others
         rop::AlphaCopy
 	rop::ConstantColor
 
+ROP functions
+
+=over
+
+=item alpha ROP, SRC_ALPHA = undef, DST_ALPHA = undef
+
+Combines one of the alpha-supporting ROPs ( Porter-Duff snd Photoshop
+operators) with
+source and destination alpha, if defined, and returns new ROP constant. This is useful
+when blending with constant alpha is required with/over images that don't have their own
+alpha channel. Or as an additional alpha channel when using icons.
+
+=item blend ALPHA
+
+Creates a ROP that would effecively execute alpha blending of the source image
+over the destination image with ALPHA value.
+
+=back
+
 =head2 sbmp:: - system bitmaps indices
 
 See also L<Prima::StdBitmap>.
@@ -1073,7 +1197,7 @@ See also L<Prima::StdBitmap>.
 =head2 scr:: - scroll exposure results
 
 C<Widget::scroll> returns one of these.
-	
+
 	scr::Error           - failure
 	scr::NoExpose        - call resulted in no new exposed areas
 	scr::Expose          - call resulted in new exposed areas, expect a repaint
@@ -1206,6 +1330,7 @@ See L<Prima::Window/windowState>
 	ws::Normal
 	ws::Minimized
 	ws::Maximized
+	ws::Fullscreen
 
 =head1 AUTHOR
 
