@@ -1,15 +1,12 @@
 #  Created by Dmitry Karasik <dk@plab.ku.dk>
 #  Modifications by Anton Berezin <tobez@tobez.org>
 package Prima::InputLine;
-use vars qw(@ISA);
-@ISA = qw(Prima::Widget Prima::MouseScroller Prima::UndoActions Prima::BidiInput);
-
 use strict;
 use warnings;
-
-use Prima::Classes;
-use Prima::IntUtils;
+use Prima;
 use Prima::Drawable::Glyphs;
+use base qw(Prima::Widget Prima::Widget::MouseScroller Prima::Widget::UndoActions Prima::Widget::BidiInput);
+
 
 sub profile_default
 {
@@ -720,6 +717,24 @@ sub on_mousedown
 	}
 }
 
+sub on_mouseclick
+{
+	my ( $self, $btn, $mod, $x, $y, $nth) = @_;
+
+	return if $self-> {mouseTransaction};
+	return if $btn != mb::Left;
+	return if $nth < 2 || $nth > 3;
+
+	$self-> clear_event;
+	return $self-> select_all if $nth == 3;
+
+	my $offset = $self-> x2offset( $x);
+	my $caplen = $self-> {n_clusters};
+	my $l      = $self->find_word_offset($offset, 0, $caplen, -1);
+	my $r      = $self->find_word_offset($offset, 1, $caplen, 1);
+	$self-> selection( $offset + $l, $offset + $r );
+}
+
 sub new_offset
 {
 	my ( $self, $ofs) = @_;
