@@ -360,8 +360,11 @@ Widget_move_notify( Handle self, Handle child, Point * moveTo)
 static int
 slave_width( register PWidget slavePtr, register int plus)
 {
-	register int width = slavePtr-> geomSize. x + slavePtr-> geomInfo. pad.x +
-								slavePtr-> geomInfo. ipad.x + plus;
+	register int width =
+		slavePtr-> geomSize. x +
+		slavePtr-> geomInfo. pad.x +
+		slavePtr-> geomInfo. ipad.x +
+		plus;
 	if ( width < slavePtr-> sizeMin.x) width = slavePtr-> sizeMin.x;
 	if ( width > slavePtr-> sizeMax.x) width = slavePtr-> sizeMax.x;
 	return width;
@@ -370,8 +373,11 @@ slave_width( register PWidget slavePtr, register int plus)
 static int
 slave_height( register PWidget slavePtr, register int plus)
 {
-	register int height = slavePtr-> geomSize.y + slavePtr-> geomInfo. pad.y +
-								slavePtr-> geomInfo. ipad.y + plus;
+	register int height =
+		slavePtr-> geomSize.y +
+		slavePtr-> geomInfo. pad.y +
+		slavePtr-> geomInfo. ipad.y +
+		plus;
 	if ( height < slavePtr-> sizeMin.y) height = slavePtr-> sizeMin.y;
 	if ( height > slavePtr-> sizeMax.y) height = slavePtr-> sizeMax.y;
 	return height;
@@ -379,9 +385,8 @@ slave_height( register PWidget slavePtr, register int plus)
 
 static int
 XExpansion(slavePtr, cavityWidth)
-	register PWidget slavePtr;		        /* First in list of remaining slaves. */
-	int cavityWidth;			/* Horizontal space left for all
-													* remaining slaves. */
+	register PWidget slavePtr; /* First in list of remaining slaves. */
+	int cavityWidth;           /* Horizontal space left for all remaining slaves. */
 {
 	int numExpand, minExpand, curExpand;
 	int childWidth;
@@ -399,25 +404,26 @@ XExpansion(slavePtr, cavityWidth)
 
 	minExpand = cavityWidth;
 	numExpand = 0;
-	for (; slavePtr != NULL;
-		slavePtr = ( PWidget) slavePtr-> geomInfo. next) {
+	for (
+		;
+		slavePtr != NULL;
+		slavePtr = ( PWidget) slavePtr-> geomInfo. next
+	) {
 		childWidth = slave_width(slavePtr, 0);
 		if ((slavePtr-> geomInfo. side == TOP) || (slavePtr-> geomInfo. side == BOTTOM)) {
-				curExpand = (cavityWidth - childWidth)/numExpand;
-				if (curExpand < minExpand) {
-					minExpand = curExpand;
-				}
+			curExpand = (cavityWidth - childWidth)/numExpand;
+			if (curExpand < minExpand)
+				minExpand = curExpand;
 		} else {
-				cavityWidth -= childWidth;
-				if (slavePtr->geomInfo. expand) {
-					numExpand++;
-				}
+			cavityWidth -= childWidth;
+			if (slavePtr->geomInfo. expand)
+				numExpand++;
 		}
 	}
-	curExpand = cavityWidth/numExpand;
-	if (curExpand < minExpand) {
+
+	curExpand = cavityWidth / numExpand;
+	if (curExpand < minExpand) 
 		minExpand = curExpand;
-	}
 	return (minExpand < 0) ? 0 : minExpand;
 }
 /*
@@ -455,24 +461,26 @@ YExpansion(slavePtr, cavityHeight)
 
 	minExpand = cavityHeight;
 	numExpand = 0;
-	for (; slavePtr != NULL; slavePtr = (PWidget) slavePtr->geomInfo. next) {
+	for (
+		;
+		slavePtr != NULL;
+		slavePtr = (PWidget) slavePtr->geomInfo. next
+	) {
 		childHeight = slave_height(slavePtr, 0);
 		if ((slavePtr-> geomInfo. side == LEFT) || (slavePtr-> geomInfo. side == RIGHT)) {
-				curExpand = (cavityHeight - childHeight)/numExpand;
-				if (curExpand < minExpand) {
-					minExpand = curExpand;
-				}
+			curExpand = (cavityHeight - childHeight)/numExpand;
+			if (curExpand < minExpand)
+				minExpand = curExpand;
 		} else {
-				cavityHeight -= childHeight;
-				if (slavePtr-> geomInfo. expand) {
-					numExpand++;
-				}
+			cavityHeight -= childHeight;
+			if (slavePtr-> geomInfo. expand)
+				numExpand++;
 		}
 	}
+
 	curExpand = cavityHeight/numExpand;
-	if (curExpand < minExpand) {
+	if (curExpand < minExpand) 
 		minExpand = curExpand;
-	}
 	return (minExpand < 0) ? 0 : minExpand;
 }
 
@@ -523,24 +531,28 @@ Widget_pack_slaves( Handle self)
 	*/
 
 	width = height = maxWidth = maxHeight = 0;
-	for (slavePtr=masterPtr; slavePtr != NULL;
-			slavePtr = ( PWidget) slavePtr-> geomInfo. next) {
-		if ((slavePtr-> geomInfo. side == TOP) || (slavePtr-> geomInfo. side == BOTTOM)) {
-				tmp = slave_width( slavePtr, width);
-				if (tmp > maxWidth) maxWidth = tmp;
-				height += slave_height(slavePtr,0);
+	for (
+		slavePtr = masterPtr;
+		slavePtr != NULL;
+		slavePtr = ( PWidget) slavePtr-> geomInfo. next
+	) {
+		if (
+			(slavePtr-> geomInfo. side == TOP   ) ||
+			(slavePtr-> geomInfo. side == BOTTOM)
+		) {
+			tmp = slave_width( slavePtr, width);
+			if (tmp > maxWidth) maxWidth = tmp;
+			height += slave_height(slavePtr,0);
 		} else {
-				tmp = slave_height(slavePtr, height);
-				if (tmp > maxHeight) maxHeight = tmp;
-				width += slave_width(slavePtr,0);
+			tmp = slave_height(slavePtr, height);
+			if (tmp > maxHeight) maxHeight = tmp;
+			width += slave_width(slavePtr,0);
 		}
 	}
-	if (width > maxWidth) {
+	if (width > maxWidth)
 		maxWidth = width;
-	}
-	if (height > maxHeight) {
+	if (height > maxHeight)
 		maxHeight = height;
-	}
 
 	/*
 	* If the total amount of space needed in the parent window has
@@ -550,9 +562,12 @@ Widget_pack_slaves( Handle self)
 	* resize us.
 	*/
 
-	if ((((maxWidth != my-> get_geomWidth(self)))
-				|| (maxHeight != my-> get_geomHeight(self)))
-				&& is_opt( optPackPropagate)) {
+	if (
+		is_opt( optPackPropagate) && (
+			maxWidth  != my-> get_geomWidth(self) ||
+			maxHeight != my-> get_geomHeight(self)
+		)
+	) {
 		Point p, oldsize;
 		p. x = maxWidth;
 		p. y = maxHeight;
@@ -560,9 +575,9 @@ Widget_pack_slaves( Handle self)
 		my-> set_geomSize( self, p);
 		size = my-> get_size( self);
 		/* if size didn't change, that means, that no cmSize came, and thus
-			the actual repacking of slaves never took place */
+		the actual repacking of slaves never took place */
 		if ( oldsize. x != size. x || oldsize. y != size. y)
-				return;
+			return;
 	} else {
 		size = my-> get_size( self);
 	}
@@ -581,62 +596,69 @@ Widget_pack_slaves( Handle self)
 	cavityX = cavityY = x = y = 0;
 	cavityWidth = size. x;
 	cavityHeight = size. y;
-	for ( slavePtr=masterPtr; slavePtr != NULL; slavePtr = ( PWidget) slavePtr-> geomInfo. next) {
-		if ((slavePtr-> geomInfo. side == TOP) || (slavePtr-> geomInfo. side == BOTTOM)) {
-				frameWidth = cavityWidth;
-				frameHeight = slave_height(slavePtr,0);
-				if (slavePtr-> geomInfo. expand)
-					frameHeight += YExpansion(slavePtr, cavityHeight);
-				cavityHeight -= frameHeight;
-				if (cavityHeight < 0) {
-					frameHeight += cavityHeight;
-					cavityHeight = 0;
-				}
-				frameX = cavityX;
-				if (slavePtr-> geomInfo. side == BOTTOM) {
-					frameY = cavityY;
-					cavityY += frameHeight;
-				} else {
-					frameY = cavityY + cavityHeight;
-				}
-		} else {
-				frameHeight = cavityHeight;
-				frameWidth = slave_width(slavePtr,0);
-				if (slavePtr->  geomInfo. expand)
-					frameWidth += XExpansion(slavePtr, cavityWidth);
-				cavityWidth -= frameWidth;
-				if (cavityWidth < 0) {
-					frameWidth += cavityWidth;
-					cavityWidth = 0;
-				}
+	for (
+		slavePtr = masterPtr;
+		slavePtr != NULL;
+		slavePtr = ( PWidget) slavePtr-> geomInfo. next
+	) {
+		if (
+			(slavePtr-> geomInfo. side == TOP   ) ||
+			(slavePtr-> geomInfo. side == BOTTOM)
+		) {
+			frameWidth = cavityWidth;
+			frameHeight = slave_height(slavePtr,0);
+			if (slavePtr-> geomInfo. expand)
+				frameHeight += YExpansion(slavePtr, cavityHeight);
+			cavityHeight -= frameHeight;
+			if (cavityHeight < 0) {
+				frameHeight += cavityHeight;
+				cavityHeight = 0;
+			}
+			frameX = cavityX;
+			if (slavePtr-> geomInfo. side == BOTTOM) {
 				frameY = cavityY;
-				if (slavePtr-> geomInfo. side == LEFT) {
-					frameX = cavityX;
-					cavityX += frameWidth;
-				} else {
-					frameX = cavityX + cavityWidth;
-				}
+				cavityY += frameHeight;
+			} else {
+				frameY = cavityY + cavityHeight;
+			}
+		} else {
+			frameHeight = cavityHeight;
+			frameWidth = slave_width(slavePtr,0);
+			if (slavePtr->  geomInfo. expand)
+				frameWidth += XExpansion(slavePtr, cavityWidth);
+			cavityWidth -= frameWidth;
+			if (cavityWidth < 0) {
+				frameWidth += cavityWidth;
+				cavityWidth = 0;
+			}
+			frameY = cavityY;
+			if (slavePtr-> geomInfo. side == LEFT) {
+				frameX = cavityX;
+				cavityX += frameWidth;
+			} else {
+				frameX = cavityX + cavityWidth;
+			}
 		}
 
 		/*
-			* Now that we've got the size of the frame for the window,
-			* compute the window's actual size and location using the
-			* fill, padding, and frame factors.
-			*/
+		* Now that we've got the size of the frame for the window,
+		* compute the window's actual size and location using the
+		* fill, padding, and frame factors.
+		*/
 
 		borderX = slavePtr-> geomInfo. pad.x;
 		borderY = slavePtr-> geomInfo. pad.y;
 		width = slavePtr->  geomSize. x + slavePtr-> geomInfo. ipad.x;
 		if (slavePtr->  geomInfo. fillx || (width > (frameWidth - borderX)))
-				width = frameWidth - borderX;
+			width = frameWidth - borderX;
 		height = slavePtr->  geomSize. y + slavePtr-> geomInfo. ipad.y;
 		if (slavePtr->  geomInfo. filly || (height > (frameHeight - borderY)))
-				height = frameHeight - borderY;
+			height = frameHeight - borderY;
 		borderX /= 2;
 		borderY /= 2;
-		if ( width < slavePtr-> sizeMin.x) width = slavePtr-> sizeMin.x;
+		if ( width  < slavePtr-> sizeMin.x) width  = slavePtr-> sizeMin.x;
 		if ( height < slavePtr-> sizeMin.y) height = slavePtr-> sizeMin.y;
-		if ( width > slavePtr-> sizeMax.x) width = slavePtr-> sizeMax.x;
+		if ( width  > slavePtr-> sizeMax.x) width  = slavePtr-> sizeMax.x;
 		if ( height > slavePtr-> sizeMax.y) height = slavePtr-> sizeMax.y;
 		switch (slavePtr-> geomInfo. anchorx) {
 		case WEST:
@@ -786,18 +808,18 @@ Widget_packInfo( Handle self, Bool set, SV * packInfo)
 		switch ( p-> anchorx) {
 		case WEST:
 			pset_c( anchor,
-					(( p-> anchory == NORTH) ? "nw" : (( p-> anchory == CENTER) ? "w" : "sw"))
-					);
+				(( p-> anchory == NORTH) ? "nw" : (( p-> anchory == CENTER) ? "w" : "sw"))
+			);
 			break;
 		case CENTER:
 			pset_c( anchor,
-					(( p-> anchory == NORTH) ? "n" : (( p-> anchory == CENTER) ? "center" : "s"))
-					);
+				(( p-> anchory == NORTH) ? "n" : (( p-> anchory == CENTER) ? "center" : "s"))
+			);
 			break;
 		case EAST:
 			pset_c( anchor,
-					(( p-> anchory == NORTH) ? "ne" : (( p-> anchory == CENTER) ? "e" : "se"))
-					);
+				(( p-> anchory == NORTH) ? "ne" : (( p-> anchory == CENTER) ? "e" : "se"))
+			);
 			break;
 		}
 
@@ -887,10 +909,10 @@ Widget_packInfo( Handle self, Bool set, SV * packInfo)
 				croak("%s: invalid 'anchor'", "Prima::Widget::pack");
 		}
 
-		if ( pexist( ipad)) var-> geomInfo. ipad. x = var-> geomInfo. ipad. y = pget_i( ipad);
+		if ( pexist( ipad))  var-> geomInfo. ipad. x = var-> geomInfo. ipad. y = pget_i( ipad);
 		if ( pexist( ipadx)) var-> geomInfo. ipad. x = pget_i( ipadx);
 		if ( pexist( ipady)) var-> geomInfo. ipad. y = pget_i( ipady);
-		if ( pexist( pad)) var-> geomInfo. pad. x = var-> geomInfo. pad. y = pget_i( pad);
+		if ( pexist( pad))   var-> geomInfo. pad. x  = var-> geomInfo. pad. y = pget_i( pad);
 		if ( pexist( padx))  var-> geomInfo. pad. x  = pget_i( padx);
 		if ( pexist( pady))  var-> geomInfo. pad. y  = pget_i( pady);
 
@@ -1051,8 +1073,11 @@ Widget_place_slaves( Handle self)
 	masterWidth  = size. x;
 	masterHeight = size. y;
 
-	for (slave=master; slave != NULL;
-			slave = ( PWidget) slave-> geomInfo. next) {
+	for (
+		slave = master;
+		slave != NULL;
+		slave = ( PWidget) slave-> geomInfo. next
+	) {
 		Point sz;
 		register GeomInfo* slavePtr = &slave-> geomInfo;
 
@@ -1068,42 +1093,42 @@ Widget_place_slaves( Handle self)
 		y1 = slavePtr->y + (slavePtr->relY*masterHeight);
 		y = (int) (y1 + ((y1 > 0) ? 0.5 : -0.5));
 		if (slavePtr-> use_w || slavePtr-> use_rw) {
-				width = 0;
-				if (slavePtr-> use_w) {
-					width += slave->geomSize.x;
-				}
-				if (slavePtr-> use_rw) {
-					/*
-					* The code below is a bit tricky.  In order to round
-					* correctly when both relX and relWidth are specified,
-					* compute the location of the right edge and round that,
-					* then compute width.  If we compute the width and round
-					* it, rounding errors in relX and relWidth accumulate.
-					*/
+			width = 0;
+			if (slavePtr-> use_w) {
+				width += slave->geomSize.x;
+			}
+			if (slavePtr-> use_rw) {
+				/*
+				* The code below is a bit tricky.  In order to round
+				* correctly when both relX and relWidth are specified,
+				* compute the location of the right edge and round that,
+				* then compute width.  If we compute the width and round
+				* it, rounding errors in relX and relWidth accumulate.
+				*/
 
-					x2 = x1 + (slavePtr->relWidth*masterWidth);
-					tmp = (int) (x2 + ((x2 > 0) ? 0.5 : -0.5));
-					width += tmp - x;
-				}
+				x2 = x1 + (slavePtr->relWidth*masterWidth);
+				tmp = (int) (x2 + ((x2 > 0) ? 0.5 : -0.5));
+				width += tmp - x;
+			}
 		} else {
 				width = sz. x;
 		}
 		if (slavePtr-> use_h || slavePtr-> use_rh) {
-				height = 0;
-				if (slavePtr->use_h) {
-					height += slave->geomSize. y;
-				}
-				if (slavePtr->use_rh) {
-					/*
-					* See note above for rounding errors in width computation.
-					*/
+			height = 0;
+			if (slavePtr->use_h) {
+				height += slave->geomSize. y;
+			}
+			if (slavePtr->use_rh) {
+				/*
+				* See note above for rounding errors in width computation.
+				*/
 
-					y2 = y1 + (slavePtr->relHeight*masterHeight);
-					tmp = (int) (y2 + ((y2 > 0) ? 0.5 : -0.5));
-					height += tmp - y;
-				}
+				y2 = y1 + (slavePtr->relHeight*masterHeight);
+				tmp = (int) (y2 + ((y2 > 0) ? 0.5 : -0.5));
+				height += tmp - y;
+			}
 		} else {
-				height = sz. y;
+			height = sz. y;
 		}
 
 		/*
@@ -1156,18 +1181,18 @@ Widget_placeInfo( Handle self, Bool set, SV * placeInfo)
 		switch ( p-> anchorx) {
 		case WEST:
 			pset_c( anchor,
-					(( p-> anchory == NORTH) ? "nw" : (( p-> anchory == CENTER) ? "w" : "sw"))
-					);
+				(( p-> anchory == NORTH) ? "nw" : (( p-> anchory == CENTER) ? "w" : "sw"))
+			);
 			break;
 		case CENTER:
 			pset_c( anchor,
-					(( p-> anchory == NORTH) ? "n" : (( p-> anchory == CENTER) ? "center" : "s"))
-					);
+				(( p-> anchory == NORTH) ? "n" : (( p-> anchory == CENTER) ? "center" : "s"))
+			);
 			break;
 		case EAST:
 			pset_c( anchor,
-					(( p-> anchory == NORTH) ? "ne" : (( p-> anchory == CENTER) ? "e" : "se"))
-					);
+				(( p-> anchory == NORTH) ? "ne" : (( p-> anchory == CENTER) ? "e" : "se"))
+			);
 			break;
 		}
 
