@@ -316,7 +316,7 @@ extern Bool img_put( Handle dest, Handle src, int dstX, int dstY, int srcX, int
 extern Bool img_bar( Handle dest, int x, int y, int w, int h, PImgPaintContext ctx);
 extern void img_integral_rotate( Handle self, Byte * new_data, int new_line_size, int degrees);
 extern Bool img_generic_rotate( Handle self, float degrees, PImage output, ColorPixel fill);
-extern Bool img_2d_transform( Handle self, float *matrix, ColorPixel fill, PImage output);
+extern Bool img_2d_transform( Handle self, Matrix matrix, ColorPixel fill, PImage output);
 extern void img_mirror( Handle self, Bool vertically);
 extern Bool img_mirror_raw( int type, int w, int h, Byte * data, Bool vertically);
 extern void img_premultiply_alpha_constant( Handle self, int alpha);
