@@ -73,6 +73,8 @@ typedef HANDLE SOCKETHANDLE;
 #define WM_REPAINT_LAYERED                ( WM_USER + 21)
 #define WM_DRAG_RESPONSE                  ( WM_USER + 22)
 #define WM_XMOUSECLICK                    ( WM_USER + 23)
+#define WM_SIGNAL                         ( WM_USER + 24)
+#define WM_SYNTHETIC_EVENT                ( WM_USER + 25)
 #define WM_TERMINATE                      ( WM_USER + 99)
 #define WM_FIRST_USER_MESSAGE             ( WM_USER +100)
 #define WM_LAST_USER_MESSAGE              ( WM_USER +900)
@@ -184,6 +186,9 @@ typedef struct _HandleOptions_ {
 	unsigned aptClipByChildren       : 1;       // cached clipping by children
 	unsigned aptIgnoreSizeMessages   : 1;       // during window recreation
 	unsigned aptGDIPlus              : 1;       // uses GDI+
+	unsigned aptWantWorldTransform   : 1;       // SetWorldTransform is wanted for text
+	unsigned aptUsedWorldTransform   : 1;       // SetWorldTransform(matrix) mode is on currently
+	unsigned aptCachedWorldTransform : 1;       // SetWorldTransform doesn't need to be called repeatedly
 } HandleOptions;
 
 #define CLIPBOARD_MAIN 0
@@ -252,6 +257,17 @@ typedef struct _WinGuts
 	Bool           application_stop_signal;
 	long           apc_error;
 	Bool           wc2mb_is_fragile;     // cannot properly process current ACP
+
+	int            get_pixel_needs_emulation; // 0 - not tried, -1 - no, 1 - yes
+	HDC            get_pixel_dc_src, get_pixel_dc_dst;
+
+	unsigned long  program_start_ts ;            // epoch
+	unsigned int   mouse_double_click_delay;     // ms
+	unsigned long  last_mouse_click_ts;          // (time - epoch) * 1000 + msec
+	HWND           last_mouse_click_source;
+	LPARAM         last_mouse_click_position;
+	unsigned int   last_mouse_click_number;
+	Byte           last_mouse_click_fingerprint; // kmXXX | mbXXX
 } WinGuts, *PWinGuts;
 
 typedef struct _WindowData
@@ -264,6 +280,7 @@ typedef struct _WindowData
 	Handle         old_foc;
 	HWND           old_active;
 	PHash          effects;
+	WINDOWPLACEMENT fs_saved_placement;
 } WindowData;
 
 typedef struct _TimerData
@@ -426,6 +443,7 @@ typedef struct _PaintState
 		RQBrush    rq_brush;
 		PDCFont    dc_font;
 		float      font_sin, font_cos;
+		Bool       wt_want, wt_used, wt_cached;
 	} paint;
 	struct {
 		HPALETTE   palette;
@@ -755,6 +773,7 @@ extern void         socket_rehash( void);
 extern Bool         select_pen(Handle self);
 extern Bool         select_brush(Handle self);
 extern Bool         select_gp_brush(Handle self);
+extern Bool         select_world_transform(Handle self, Bool want_transform);
 extern void         stylus_clean( void);
 extern PDCObject    stylus_fetch( void * key );
 extern Bool         stylus_is_complex(Handle self);
