@@ -241,10 +241,14 @@ init_x11( char * error_buf )
 		"XdndActionList",
 	 	"XdndActionDescription",
 		"text/plain",
-		"_NET_WM_ICON"
+		"_NET_WM_ICON",
+		"_NET_FRAME_EXTENTS",
+		"_NET_WM_STATE_FULLSCREEN"
 	};
 	char hostname_buf[256], *hostname = hostname_buf, *env;
 
+	XInitThreads();
+
 	guts. click_time_frame = 200;
 	guts. double_click_time_frame = 200;
 	guts. visible_timeout = 500;
@@ -1012,7 +1016,7 @@ apc_application_get_size( Handle self)
 Box *
 apc_application_get_monitor_rects( Handle self, int * nrects)
 {
-#if defined(HAVE_X11_EXTENSIONS_XRANDR_H) && (RANDR_MAJOR > 1 || (RANDR_MAJOR == 1 && RANDR_MINOR > 3))
+#if defined(HAVE_X11_EXTENSIONS_XRANDR_H) && defined(HAVE_XRANDR_1_5)
 	XRRMonitorInfo *m;
 	Box * ret = NULL;
 	int n;
