@@ -824,7 +824,7 @@ process_wm_sync_data( Handle self, WMSyncData * wmsd)
 		if ( PObject( self)-> stage == csDead) return false;
 	}
 
-	if ( size_changed && XX-> flags. want_visible && !guts. net_wm_maximization) {
+	if ( size_changed && XX-> flags. want_visible && !guts. net_wm_maximization && !XX-> flags. fullscreen) {
 		int qx = guts. displaySize.x * 4 / 5, qy = guts. displaySize.y * 4 / 5;
 		bzero( &e, sizeof( Event));
 		if ( !XX-> flags. zoomed) {
@@ -951,7 +951,12 @@ wm_event( Handle self, XEvent *xev, PEvent ev)
 		}
 		break;
 	case PropertyNotify:
-		if ( xev-> xproperty. atom == NET_WM_STATE && xev-> xproperty. state == PropertyNewValue) {
+		if (
+			xev-> xproperty. atom == NET_WM_STATE &&
+			xev-> xproperty. state == PropertyNewValue &&
+			!X(self)-> flags. fullscreen &&
+			X(self)-> flags. mapped
+		) {
 			DEFXX;
 			ev-> cmd = cmWindowState;
 			ev-> gen. source = self;
@@ -965,6 +970,9 @@ wm_event( Handle self, XEvent *xev, PEvent ev)
 				if ( XX-> flags. zoomed) {
 					ev-> gen. i = wsNormal;
 					XX-> flags. zoomed = 0;
+				} else if ( XX-> flags. iconic ) {
+					ev-> gen. i = wsNormal;
+					XX-> flags. iconic = 0;
 				} else
 					ev-> cmd = 0;
 			}
@@ -1291,9 +1299,10 @@ prima_handle_event( XEvent *ev, XEvent *next_event)
 			bev-> button == guts.last_button_event.button &&
 			bev-> button != guts. mouse_wheel_up &&
 			bev-> button != guts. mouse_wheel_down &&
-			bev-> time - guts.last_button_event.time <= guts.click_time_frame) {
-	  		e. cmd = cmMouseClick;
-			e. pos. dblclk = true;
+			bev-> time - guts.last_button_event.time <= guts.click_time_frame
+		) {
+			e. cmd = cmMouseClick;
+			e. pos. nth = ++guts.last_mouseclick_number;
 		}
 
 		if (
@@ -1320,6 +1329,7 @@ prima_handle_event( XEvent *ev, XEvent *next_event)
 			secondary. pos. mod = e. pos. mod;
 			secondary. pos. button = e. pos. button;
 			memcpy( &guts.last_click, bev, sizeof(guts.last_click));
+			guts. last_mouseclick_number = 1;
 			if ( e. pos. button == mbRight) {
 				Event ev;
 				bzero( &ev, sizeof(ev));
@@ -1330,18 +1340,9 @@ prima_handle_event( XEvent *ev, XEvent *next_event)
 				apc_message( self, &ev, true);
 				if ( PObject( self)-> stage == csDead) return;
 			}
-		} else if ( e.cmd == cmMouseUp &&
-			(guts.last_button_event.type == (cmMouseClick | 0x8000)) &&
-			bev-> window == guts.last_button_event.window &&
-			bev-> button == guts.last_button_event.button &&
-			bev-> time - guts.last_button_event.time <= guts.click_time_frame
-		) {
-			e.pos.dblclk = 1;
 		}
 		memcpy( &guts.last_button_event, bev, sizeof(*bev));
 		guts. last_button_event.type = e.cmd;
-		if (e.pos.dblclk)
-			guts.last_button_event.type |= 0x8000;
 
 		if ( e. cmd == cmMouseDown) {
 			if ( XX-> flags. first_click) {
@@ -2012,9 +2013,11 @@ send_pending_events( void)
 	if ( !prima_guts.application ) return 0;
 
 	for ( pe = TAILQ_FIRST( &guts.peventq); pe != NULL; ) {
-		next = TAILQ_NEXT( pe, peventq_link);
-		if (( stage = PComponent( pe->recipient)-> stage) != csConstructing) {
+		next  = TAILQ_NEXT( pe, peventq_link);
+		stage = PComponent( pe->recipient)-> stage;
+		if ( stage != csConstructing) { /* or not yet there */
 			TAILQ_REMOVE( &guts.peventq, pe, peventq_link);
+			unprotect_object( pe-> recipient );
 		}
 		if ( stage == csNormal) {
 			apc_message( pe-> recipient, &pe-> event, false);
@@ -2265,6 +2268,7 @@ prima_one_loop_round( int wait, Bool careOfApplication)
 	if ( x_events_pending && ( prima_guts.application || !careOfApplication) )
 		x_flush();
 	handle_queued_events(careOfApplication);
+	exception_dispatch_pending_signals();
 
 	return prima_guts.application != NULL_HANDLE;
 }
@@ -2289,6 +2293,7 @@ apc_message( Handle self, PEvent e, Bool is_post)
 		if (!( pe = alloc1(PendingEvent))) return false;
 		memcpy( &pe->event, e, sizeof(pe->event));
 		pe-> recipient = self;
+		protect_object(self);
 		TAILQ_INSERT_TAIL( &guts.peventq, pe, peventq_link);
 	} else {
 		guts. total_events++;
